@@ -134,6 +134,7 @@ struct Resolver<'a> {
     ident_type: IdentType,
     handle_types: bool,
     in_type: bool,
+    in_ts_module: bool,
 }
 
 impl<'a> Resolver<'a> {
@@ -145,6 +146,7 @@ impl<'a> Resolver<'a> {
             ident_type: IdentType::Ref,
             handle_types,
             in_type: false,
+            in_ts_module: false,
         }
     }
 
@@ -417,7 +419,14 @@ impl<'a> VisitMut for Resolver<'a> {
         folder.ident_type = old;
         folder.hoist = old_hoist;
 
-        e.body.visit_mut_with(&mut folder);
+        {
+            folder.hoist = false;
+
+            match &mut e.body {
+                BlockStmtOrExpr::BlockStmt(s) => s.stmts.visit_mut_with(&mut folder),
+                BlockStmtOrExpr::Expr(e) => e.visit_mut_with(&mut folder),
+            }
+        }
 
         e.return_type.visit_mut_with(&mut folder);
     }
@@ -583,7 +592,9 @@ impl<'a> VisitMut for Resolver<'a> {
             Scope::new(ScopeKind::Fn, Some(&self.current)),
             self.handle_types,
         );
+
         if let Some(ident) = &mut e.ident {
+            self.in_type = false;
             folder.visit_mut_binding_ident(ident, None)
         }
         e.function.visit_mut_with(&mut folder);
@@ -774,6 +785,26 @@ impl<'a> VisitMut for Resolver<'a> {
         }
     }
 
+    fn visit_mut_setter_prop(&mut self, n: &mut SetterProp) {
+        n.key.visit_mut_with(self);
+
+        {
+            let child_mark = Mark::fresh(self.mark);
+
+            // Child folder
+            let mut child = Resolver::new(
+                child_mark,
+                Scope::new(ScopeKind::Fn, Some(&self.current)),
+                self.handle_types,
+            );
+
+            child.in_type = false;
+            child.ident_type = IdentType::Binding;
+            n.param.visit_mut_with(&mut child);
+            n.body.visit_mut_with(&mut child);
+        };
+    }
+
     fn visit_mut_method_prop(&mut self, m: &mut MethodProp) {
         m.key.visit_mut_with(self);
 
@@ -792,7 +823,7 @@ impl<'a> VisitMut for Resolver<'a> {
     }
 
     fn visit_mut_module_items(&mut self, stmts: &mut Vec<ModuleItem>) {
-        if self.current.kind != ScopeKind::Fn {
+        if !self.in_ts_module && self.current.kind != ScopeKind::Fn {
             return stmts.visit_mut_children_with(self);
         }
 
@@ -1040,6 +1071,7 @@ impl<'a> VisitMut for Resolver<'a> {
             Scope::new(ScopeKind::Block, Some(&self.current)),
             self.handle_types,
         );
+        child_folder.in_ts_module = true;
 
         decl.body.visit_mut_children_with(&mut child_folder);
     }
@@ -1213,6 +1245,9 @@ impl VisitMut for Hoister<'_, '_> {
     #[inline]
     fn visit_mut_function(&mut self, _: &mut Function) {}
 
+    #[inline]
+    fn visit_mut_setter_prop(&mut self, _: &mut SetterProp) {}
+
     fn visit_mut_var_decl(&mut self, node: &mut VarDecl) {
         if self.in_block {
             match node.kind {
